#include "mem.h"
#include "roms.h"
#include "input.h"

uint32 texturePool = 0x700000;
uint32 texture;
/* I borrowed this function from Dan Potter just like everyone else :) */

void *malloc_base = (void *)0x8c100000;			/* DC RAM begins here */

void *malloc(int size)
{
	void *rv = malloc_base;
	malloc_base += size;
	return rv;
}

uint32 textureMalloc(uint32 size) {
	uint32 rv = texturePool;

	/* Align to 8 bytes */
	size = (size + 7) & (~7);
	texturePool += size;

	return rv;
}

int allocateMem()
{
	int i;
	texture=0;
	pBitmap1 = malloc(0x40000);				/* 256*256*4 bytes */
	pBitmap2 = malloc(0x40000);
	pBitmap3 = malloc(0x40000);
	pBlankBitmap = malloc(0x4000);
	pRAM = malloc(0x8000);					/* 16k for main RAM (1st 8k for ROMs) */
	pCharA = malloc(0x1000);				/* 2k for 8x8 Character Map A */
	pCharB = malloc(0x1000);				/* 2k for 8x8 Character Map B */
	pVRAM0 = malloc(0x1000);				/* 2k for VRAM bank 0 */
	pVRAM1 = malloc(0x1000);				/* 2k for VRAM bank 1 */

	if (!pRAM || !pCharA || !pCharB || !pVRAM0 || !pVRAM1)
	{
		return 0;						/* Fail if malloc didnt work */
	}

	/* Setup some default values for our RAM */
	for (i=0x0000; i < 0x8000; i++) pRAM[i]=0;
	for (i=0x0000; i < 0x1000; i++)
	{
		pVRAM0[i]=0;
		pVRAM1[i]=0;
	}

	/* Clear our bitmaps */

	clearBitmaps();
	textureSetup();

	return 1;
}

void textureSetup() {
	uint32 ts = textureMalloc(256*256); /* allocate the memory for a 256x256 texture */
	pVram = (uint16*)(0xa4000000 + ts); /* set vram to 0x4000000 + how much memory the texture requires */	
      texture = ts;
}
      	
void clearBitmaps()
{
	int x,y;
	for (x = 0; x < 256; x++)
		for (y = 0; y < 8; y++)
			pBlankBitmap[x+(y<<8)] = 0;
		for (y = 0; y < 256; y++)
		{
			pBitmap1[x+(y<<8)] = 0;
			pBitmap2[x+(y<<8)] = 0;
			pBitmap3[x+(y<<8)] = 0;
		}
}

void loadRom(byte *from, byte *to, int start, int end)
{
	int i;
	int j=0x0000;
	for (i = start; i < end; i++)
	{
		to[i] = from[j];
		j++;
	}
}

void loadGame()
{
	loadRom(ic45,pRAM,0x0000,0x0800);
	loadRom(ic46,pRAM,0x0800,0x1000);
	loadRom(ic47,pRAM,0x1000,0x1800);
	loadRom(ic48,pRAM,0x1800,0x2000);
	loadRom(ic49,pRAM,0x2000,0x2800);
	loadRom(ic50,pRAM,0x2800,0x3000);
	loadRom(ic51,pRAM,0x3000,0x3800);
	loadRom(ic52,pRAM,0x3800,0x4000);
	loadRom(ic39,pCharA,0x0000,0x0800);
	loadRom(ic40,pCharA,0x0800,0x1000);
	loadRom(ic23,pCharB,0x0000,0x0800);
	loadRom(ic24,pCharB,0x0800,0x1000);
}

unsigned Z80_RDMEM(dword A)
{
	if (A < 0x4000) return pRAM[A];
	if (A >= 0x4000 && A <= 0x43ff)
	{		/* CharA */
		if (vramBank) return pVRAM1[A-0x4000];	/* Handle VRAM banking */
		else return pVRAM0[A-0x4000];
	}
	if (A >= 0x4800 && A <= 0x4bff)
	{		/* CharB */
		if (vramBank) return pVRAM1[A-0x4000];	/* Handle VRAM banking */
		else return pVRAM0[A-0x4000];
	}
	if (A >= 0x7000 && A <= 0x73ff)
	{		/* Controls */
		gameInput();					/* Read Controller Input */
		return pRAM[0x7000];
	}
	if (A >= 0x7800 && A <= 0x7bff)
	{		/* Dip Switches */
		if (pRAM[0x7800]&0x80)
		{							/* If VBlank bit is set... */
			pRAM[0x7800]&=~0x80;			/* Unset it */
			return (pRAM[0x7800]|0x80);
		}
		return pRAM[0x7800];
	}
	return pRAM[A];
}

void Z80_WRMEM(dword A, byte V)
{
	if (A >= 0x4000 && A <= 0x43ff)
	{		/* CharA */
		if (A < 0x4340)					/* Only lower 32x26 bytes for VRAM */
			bufferCharA[A-0x4000] = 1;		/* that byte needs to be redrawn */
		if (vramBank) pVRAM1[A-0x4000] = V;		/* Handle VRAM banking */
		else pVRAM0[A-0x4000] = V;
	}
	else if (A >= 0x4800 && A <= 0x4bff)
	{		/* CharB */
		if (A < 0x4b40)					/* Only lower 32x26 bytes for VRAM */
			bufferCharB[A-0x4800] = 1;		/* that byte need to be redrawn */
		if (vramBank) pVRAM1[A-0x4000] = V;		/* Handle VRAM banking */
		else pVRAM0[A-0x4000] = V;
	}
	/* This checks to see if the 1st bit of pRAM[0x5000] needs to be changed */
	/* Bit 1 of 0x5000 is used to set the current VRAM bank */
	else if (A >= 0x5000 && A <= 0x53ff && (pRAM[0x5000]&1)!=(V&1))
	{
		int i;
		if (V&1) vramBank = 1;
		else vramBank = 0;
		/* Force full redraw */
		for (i = 0; i < 832; i++)
		{
			bufferCharA[i] = 1;
			bufferCharB[i] = 1;
		}
		/* Set 0x5000 bit 1 to current vram bank */
		pRAM[0x5000] = (pRAM[0x5000]&~1)+(V&1);
	}
	/* This checks to see if the 2nd bit of pRAM[0x5000] needs to be changed */
	/* Bit 2 of 0x5000 is used to set the current palette */
	else if (A >= 0x5000 && A <= 0x53ff && (pRAM[0x5000]&2)!=(V&2))
	{
		if (V&2) palette = 2;
		else palette = 1;
		pRAM[0x5000] = (pRAM[0x5000]&~2)+(V&2);
	}
	/* Mirroring the scroll buffer... */
	else if (A >= 0x5800 && A <= 0x5bff) pRAM[0x5800] = V;
	else if (A < 0x4000 || A >= 0x8000) return;	/* ERROR: Trying to write to ROM */
	pRAM[A] = V;
	return;
}