#include "global.h"
#include "palette.h"
#include "strqueues.h"

void drawBitmap1(int pos)
{
	int x = 25-(pos>>5);    /* Get the X and Y coords from the pos */
	int y = pos&31;

	unsigned int cx, cy;    /* Setup some vars we'll need... */
	byte chr, byte1, byte2, color, color2;


	/* Get character data from VRAM, the character maps are 8x8 charsets */
	if (vramBank) chr = pVRAM1[((25-x)<<5)+y];
	else chr = pVRAM0[((25-x)<<5)+y];

	/* Get the base color (I dont really fully understand the color part yet) */
	color = 4*(chr>>5);

	for (cx = 0; cx < 8; cx++)
	{
		byte1 = pCharA[(chr<<3)+cx];
		byte2 = pCharA[(chr<<3)+cx+0x0800];
		for (cy = 0; cy < 8; cy++)
		{
			color2 = color+2*((byte1>>cy)&1);
			color2 += (byte2>>cy)&1;
			if ((color2&3)==0) color2 = 0;
				pBitmap1[((x<<3)+(7-cx))+((((y<<3)+cy))<<8)] = getColor(color2);
		}
	}
}

void drawBitmap2(int pos)
{
	int x = 25-(pos>>5);
	int y = pos&31;

	unsigned int cx, cy;
	byte chr, byte1, byte2, color, color2;

	if (vramBank) chr = pVRAM1[0x0800+((25-x)<<5)+y];
	else chr = pVRAM0[0x0800+((25-x)<<5)+y];

	color = 4*(chr>>5)+32;

	for (cx = 0; cx < 8; cx++)
	{
		byte1 = pCharB[(chr<<3)+cx];
		byte2 = pCharB[(chr<<3)+cx+0x0800];
		for (cy = 0; cy < 8; cy++)
		{
      		color2 = color+2*((byte1>>cy)&1);
			color2 += (byte2>>cy)&1;

			pBitmap2[((x<<3)+(7-cx))+((((y<<3)+cy))<<8)] = getColor(color2);
		}
	}
}

int getColor(int color)
{
	if (palette) return phoenixPalette1[color];
	else if (palette == 2) return phoenixPalette2[color];
	else return 0;
}

typedef struct {
	float   x, y, z;
	float   u, v;
} polyPlace_t;

polyPlace_t polys = { 128.0f, 0.0f, 1.0f, 0.0f, 0.0f };

void drawPoly(polyPlace_t *p) {
	/* Opaque Colored vertex */
	vertex_ot_t vert;

	vert.flags = TA_VERTEX_NORMAL;
	vert.x = p->x + 0.0f; vert.y = p->y + 480.0f; vert.z = p->z;
	vert.a = 1.0f; vert.r = 1.0f; vert.g = 1.0f; vert.b = 1.0f;
	vert.u = 0.0f; vert.v = 1.0f;
	ta_commit_vertex(&vert, sizeof(vert));
	
	vert.flags = TA_VERTEX_NORMAL;
	vert.x = p->x + 0.0f; vert.y = p->y + 0.0f;
	vert.v = 0.0f;
	ta_commit_vertex(&vert, sizeof(vert));

	vert.flags = TA_VERTEX_NORMAL;
	vert.x = p->x + 432.0f; vert.y = p->y + 480.0f;
	vert.u = 1.0f; vert.v = 1.0f;
	ta_commit_vertex(&vert, sizeof(vert));

	vert.flags = TA_VERTEX_EOL;
	vert.x = p->x + 432.0f; vert.y = p->y + 0.0f;
	vert.v = 0.0f;
	ta_commit_vertex(&vert, sizeof(vert));
}

void drawScreen()
{
	int x,y,pos,temp,sourcey,desty;
	byte scroll = pRAM[0x5800];
      poly_hdr_t poly;
	
	
	for (pos = 0; pos < 832; pos++)
	{
		if (bufferCharA[pos])
		{
			bufferCharA[pos] = 0;
			drawBitmap1(pos);
		}
		if (bufferCharB[pos])
		{
			bufferCharB[pos] = 0;
			drawBitmap2(pos);
		}
	}

	temp=256-scroll;

	for (y=0; y<8; y++)
	{
		sqmemcpy(pBitmap3+(y<<8), pBlankBitmap+(y<<8));
	}
		
	sourcey = scroll;
	for (y=8; y<temp; y++)
	{
		sqmemcpy(pBitmap3+(y<<8), pBitmap2+(sourcey<<8));
		sourcey++;
	}

	desty = temp;    
	for (y=0; y<scroll; y++)
	{
		sqmemcpy(pBitmap3+(desty<<8), pBitmap2+(y<<8));
		desty++;
	}

	for (y=0; y<256; y++)
	{
		for (x=0; x<256; x++)
		{
			if (pBitmap1[x+(y<<8)] ==0) {}
			else pBitmap3[x+(y<<8)] = pBitmap1[x+(y<<8)];
		}
	}
  
	for (y=0; y<256; y++)
	{
		sqmemcpy(pVram+(y<<8), pBitmap3+(y<<8));
	}

	/* Send polygon header to the TA using store queues */
	ta_build_poly_hdr(&poly, TA_OPAQUE, TA_RGB565, 256, 256, texture, 0);
	ta_commit_poly_hdr(&poly);

	/* Draw all polygons */
	drawPoly(&polys);

	/* End of opaque list */
	ta_commit_eol();

	/* TA freaks out at having a completely empty list */
	ta_build_poly_hdr(&poly, TA_TRANSLUCENT, TA_NO_TEXTURE, 0, 0, 0, 0);
	ta_commit_poly_hdr(&poly);

	/* End of translucent list */
	ta_commit_eol();

	/* Finish the frame */
	ta_finish_frame();
	
	return;
}